@@ -43,6 +43,7 @@
 #include "ephy-string.h"
 #include "ephy-web-app-utils.h"
 #include "ephy-web-dom-utils.h"
+#include "ephy-web-extension-proxy.h"
 #include "ephy-zoom.h"
 
 #include <gio/gio.h>
@@ -64,8 +65,6 @@
 #define EMPTY_PAGE              _("Blank page") /* Title for the empty page */
 
 #define EPHY_PAGE_TEMPLATE_ERROR         "/org/gnome/epiphany/page-templates/error.html"
-#define EPHY_PAGE_TEMPLATE_RECOVERY      "/org/gnome/epiphany/page-templates/recovery.html"
-#define EPHY_PAGE_TEMPLATE_PROCESS_CRASH "/org/gnome/epiphany/page-templates/process-crash.html"
 
 struct _EphyWebViewPrivate {
   EphyWebViewSecurityLevel security_level;
@@ -111,6 +110,9 @@ struct _EphyWebViewPrivate {
   /* TLS information. */
   GTlsCertificate *certificate;
   GTlsCertificateFlags tls_errors;
+
+  /* Web Extension */
+  EphyWebExtensionProxy *web_extension;
 };
 
 typedef struct {
@@ -184,12 +186,11 @@ static char *
 popups_manager_new_window_info (EphyEmbedContainer *container)
 {
   EphyEmbed *embed;
-  EphyWebViewChrome chrome;
   GtkAllocation allocation;
   gboolean is_popup;
   char *features;
 
-  g_object_get (container, "chrome", &chrome, "is-popup", &is_popup, NULL);
+  g_object_get (container, "is-popup", &is_popup, NULL);
   g_return_val_if_fail (is_popup, g_strdup (""));
 
   embed = ephy_embed_container_get_active_child (container);
@@ -201,7 +202,7 @@ popups_manager_new_window_info (EphyEmbedContainer *container)
              ("width=%d,height=%d,toolbar=%d",
               allocation.width,
               allocation.height,
-              (chrome & EPHY_WEB_VIEW_CHROME_TOOLBAR) > 0);
+              1);
 
   return features;
 }
@@ -608,49 +609,70 @@ icon_changed_cb (EphyWebView *view,
   _ephy_web_view_update_icon (view);
 }
 
+typedef struct {
+  EphyWebView *web_view;
+  guint request_id;
+} FormAuthRequestData;
+
 static void
 form_auth_data_save_confirmation_response (GtkInfoBar *info_bar,
                                            gint response_id,
-                                           gpointer user_data)
+                                           FormAuthRequestData *data)
 {
-  GDBusProxy *web_extension;
-  guint request_id = GPOINTER_TO_INT (user_data);
-
   gtk_widget_destroy (GTK_WIDGET (info_bar));
 
-  web_extension = ephy_embed_shell_get_web_extension_proxy (ephy_embed_shell_get_default ());
-  if (!web_extension)
-    return;
+  if (data->web_view->priv->web_extension) {
+    ephy_web_extension_proxy_form_auth_data_save_confirmation_response (data->web_view->priv->web_extension,
+                                                                        data->request_id,
+                                                                        response_id == GTK_RESPONSE_YES);
+  }
 
-  g_dbus_proxy_call (web_extension,
-                     "FormAuthDataSaveConfirmationResponse",
-                     g_variant_new ("(ub)", request_id, response_id == GTK_RESPONSE_YES),
-                     G_DBUS_CALL_FLAGS_NONE,
-                     -1, NULL, NULL, NULL);
+  g_slice_free (FormAuthRequestData, data);
 }
 
 static void
-form_auth_data_save_requested (EphyEmbedShell *shell,
+form_auth_data_save_requested (EphyWebExtensionProxy *web_extension,
                                guint request_id,
                                guint64 page_id,
                                const char *hostname,
                                const char *username,
-                               WebKitWebView *web_view)
+                               EphyWebView *web_view)
 {
   GtkWidget *info_bar;
+  FormAuthRequestData *data;
 
-  if (webkit_web_view_get_page_id (web_view) != page_id)
-    return;
+  g_assert (webkit_web_view_get_page_id (WEBKIT_WEB_VIEW (web_view)) == page_id);
 
-  info_bar = ephy_web_view_create_form_auth_save_confirmation_info_bar (EPHY_WEB_VIEW (web_view),
-                                                                        hostname, username);
+  info_bar = ephy_web_view_create_form_auth_save_confirmation_info_bar (web_view, hostname, username);
+  data = g_slice_new (FormAuthRequestData);
+  data->web_view = web_view;
+  data->request_id = request_id;
   g_signal_connect (info_bar, "response",
                     G_CALLBACK (form_auth_data_save_confirmation_response),
-                    GINT_TO_POINTER (request_id));
+                    data);
 
   gtk_widget_show (info_bar);
 }
 
+static void
+page_created_cb (EphyEmbedShell *shell,
+                 guint64 page_id,
+                 EphyWebExtensionProxy *web_extension,
+                 EphyWebView *web_view)
+{
+  EphyWebViewPrivate *priv = web_view->priv;
+
+  if (webkit_web_view_get_page_id (WEBKIT_WEB_VIEW (web_view)) != page_id)
+    return;
+
+  priv->web_extension = web_extension;
+  g_object_add_weak_pointer (G_OBJECT (priv->web_extension), (gpointer *)&priv->web_extension);
+
+  g_signal_connect (priv->web_extension, "form-auth-data-save-requested",
+                    G_CALLBACK (form_auth_data_save_requested),
+                    web_view);
+}
+
 static void
 ephy_web_view_dispose (GObject *object)
 {
@@ -1310,6 +1332,18 @@ decide_policy_cb (WebKitWebView *web_view,
   response = webkit_response_policy_decision_get_response (response_decision);
   mime_type = webkit_uri_response_get_mime_type (response);
 
+  /* If WebKit can't handle the mime type start the download
+     process */
+  if (webkit_response_policy_decision_is_mime_type_supported (response_decision))
+    return FALSE;
+
+  /* If it's not the main resource we don't need to set the document type. */
+  request = webkit_response_policy_decision_get_request (response_decision);
+  request_uri = webkit_uri_request_get_uri (request);
+  main_resource = webkit_web_view_get_main_resource (web_view);
+  if (g_strcmp0 (webkit_web_resource_get_uri (main_resource), request_uri) != 0)
+    return FALSE;
+
   type = EPHY_WEB_VIEW_DOCUMENT_OTHER;
   if (!strcmp (mime_type, "text/html") || !strcmp (mime_type, "text/plain"))
     type = EPHY_WEB_VIEW_DOCUMENT_HTML;
@@ -1327,17 +1361,6 @@ decide_policy_cb (WebKitWebView *web_view,
     g_object_notify (G_OBJECT (web_view), "document-type");
   }
 
-  /* If WebKit can't handle the mime type start the download
-     process */
-  if (webkit_response_policy_decision_is_mime_type_supported (response_decision))
-    return FALSE;
-
-  request = webkit_response_policy_decision_get_request (response_decision);
-  request_uri = webkit_uri_request_get_uri (request);
-  main_resource = webkit_web_view_get_main_resource (web_view);
-  if (g_strcmp0 (webkit_web_resource_get_uri (main_resource), request_uri) != 0)
-    return FALSE;
-
   webkit_policy_decision_download (decision);
 
   return TRUE;
@@ -1733,6 +1756,7 @@ ephy_web_view_load_error_page (EphyWebView *view,
   char *msg;
   char *button_label;
   char *stylesheet;
+  const char *custom_class;
   GBytes *html_file;
 
   if (error)
@@ -1747,6 +1771,8 @@ ephy_web_view_load_error_page (EphyWebView *view,
   lang = g_strdup (pango_language_to_string (gtk_get_default_language ()));
   g_strdelimit (lang, "_-@", '\0');
 
+  html_file = g_resources_lookup_data (EPHY_PAGE_TEMPLATE_ERROR, 0, NULL);
+
   switch (page) {
     case EPHY_WEB_VIEW_ERROR_PAGE_NETWORK_ERROR:
       page_title = g_strdup_printf (_("Problem loading “%s”"), hostname);
@@ -1760,10 +1786,8 @@ ephy_web_view_load_error_page (EphyWebView *view,
                                "address. You may wish to verify that your "
                                "internet connection is working correctly.</p>"),
                              uri, reason);
-
-      button_label = g_strdup (_("Try Again"));
-
-      html_file = g_resources_lookup_data (EPHY_PAGE_TEMPLATE_ERROR, 0, NULL);
+      button_label = g_strdup (_("Try again"));
+      custom_class = "network-error";
       break;
     case EPHY_WEB_VIEW_ERROR_PAGE_CRASH:
       page_title = g_strdup_printf (_("Problem loading “%s”"), hostname);
@@ -1774,19 +1798,15 @@ ephy_web_view_load_error_page (EphyWebView *view,
                                "please report the problem to the "
                                "<strong>%s</strong> developers.</p>"),
                              LSB_DISTRIBUTOR);
-
       button_label = g_strdup (_("Reload Anyway"));
-
-      html_file = g_resources_lookup_data (EPHY_PAGE_TEMPLATE_RECOVERY, 0, NULL);
+      custom_class = "page-crash";
       break;
     case EPHY_WEB_VIEW_ERROR_PROCESS_CRASH:
       page_title = g_strdup_printf (_("Problem displaying “%s”"), hostname);
       msg_title = g_strdup (_("Oops!"));
       msg = g_strdup (_("Something went wrong while displaying this page. Please reload or visit a different page to continue."));
       button_label = g_strdup (_("Reload Anyway"));
-
-      html_file = g_resources_lookup_data (EPHY_PAGE_TEMPLATE_PROCESS_CRASH, 0, NULL);
-
+      custom_class = "process-crash";
       break;
     default:
       return;
@@ -1806,6 +1826,7 @@ ephy_web_view_load_error_page (EphyWebView *view,
                    page_title,
                    stylesheet,
                    uri,
+                   custom_class,
                    msg_title, msg, button_label);
 
   g_bytes_unref (html_file);
@@ -2001,8 +2022,8 @@ ephy_web_view_init (EphyWebView *web_view)
                     G_CALLBACK (ge_popup_blocked_cb),
                     NULL);
 
-  g_signal_connect (ephy_embed_shell_get_default (), "form-auth-data-save-requested",
-                    G_CALLBACK (form_auth_data_save_requested),
+  g_signal_connect (ephy_embed_shell_get_default (), "page-created",
+                    G_CALLBACK (page_created_cb),
                     web_view);
 }
 
@@ -2575,19 +2596,13 @@ ephy_web_view_set_typed_address (EphyWebView *view,
 }
 
 static void
-has_modified_forms_cb (GDBusProxy *web_extension,
+has_modified_forms_cb (EphyWebExtensionProxy *web_extension,
                        GAsyncResult *result,
                        GTask *task)
 {
-  GVariant *return_value;
-  gboolean retval = FALSE;
-
-  return_value = g_dbus_proxy_call_finish (web_extension, result, NULL);
-  if (return_value) {
-    g_variant_get (return_value, "(b)", &retval);
-    g_variant_unref (return_value);
-  }
+  gboolean retval;
 
+  retval = ephy_web_extension_proxy_web_page_has_modified_forms_finish (web_extension, result, NULL);
   g_task_return_boolean (task, retval);
   g_object_unref (task);
 }
@@ -2612,19 +2627,18 @@ ephy_web_view_has_modified_forms (EphyWebView *view,
                                   GAsyncReadyCallback callback,
                                   gpointer user_data)
 {
-  GTask *task = g_task_new (view, cancellable, callback, user_data);
-  GDBusProxy *web_extension;
+  GTask *task;
 
-  web_extension = ephy_embed_shell_get_web_extension_proxy (ephy_embed_shell_get_default ());
-  if (web_extension) {
-    g_dbus_proxy_call (web_extension,
-                       "HasModifiedForms",
-                       g_variant_new ("(t)", webkit_web_view_get_page_id (WEBKIT_WEB_VIEW (view))),
-                       G_DBUS_CALL_FLAGS_NONE,
-                       -1,
-                       cancellable,
-                       (GAsyncReadyCallback)has_modified_forms_cb,
-                       g_object_ref (task));
+  g_return_if_fail (EPHY_IS_WEB_VIEW (view));
+
+  task = g_task_new (view, cancellable, callback, user_data);
+
+  if (view->priv->web_extension) {
+    ephy_web_extension_proxy_web_page_has_modified_forms (view->priv->web_extension,
+                                                          webkit_web_view_get_page_id (WEBKIT_WEB_VIEW (view)),
+                                                          cancellable,
+                                                          (GAsyncReadyCallback)has_modified_forms_cb,
+                                                          g_object_ref (task));
   } else {
     g_task_return_boolean (task, FALSE);
   }
@@ -2642,6 +2656,151 @@ ephy_web_view_has_modified_forms_finish (EphyWebView *view,
   return g_task_propagate_boolean (G_TASK (result), error);
 }
 
+typedef struct {
+  gboolean icon_result;
+  char *icon_uri;
+  char *icon_color;
+} GetBestWebAppIconAsyncData;
+
+static void
+get_best_web_app_icon_async_data_free (GetBestWebAppIconAsyncData *data)
+{
+  g_free (data->icon_uri);
+  g_free (data->icon_color);
+
+  g_slice_free (GetBestWebAppIconAsyncData, data);
+}
+
+static void
+get_best_web_app_icon_cb (EphyWebExtensionProxy *web_extension,
+                          GAsyncResult *result,
+                          GTask *task)
+{
+  gboolean retval = FALSE;
+  char *uri = NULL;
+  char *color = NULL;
+  GError *error = NULL;
+
+  if (!ephy_web_extension_proxy_get_best_web_app_icon_finish (web_extension, result, &retval, &uri, &color, &error)) {
+    g_task_return_error (task, error);
+  } else {
+    GetBestWebAppIconAsyncData *data = g_slice_new0 (GetBestWebAppIconAsyncData);
+
+    data->icon_result = retval;
+    data->icon_uri = uri;
+    data->icon_color = color;
+    g_task_return_pointer (task, data, (GDestroyNotify)get_best_web_app_icon_async_data_free);
+  }
+  g_object_unref (task);
+}
+
+void
+ephy_web_view_get_best_web_app_icon (EphyWebView *view,
+                                     GCancellable *cancellable,
+                                     GAsyncReadyCallback callback,
+                                     gpointer user_data)
+{
+  GTask *task;
+
+  g_return_if_fail (EPHY_IS_WEB_VIEW (view));
+
+  task = g_task_new (view, cancellable, callback, user_data);
+
+  if (view->priv->web_extension) {
+    ephy_web_extension_proxy_get_best_web_app_icon (view->priv->web_extension,
+                                                    webkit_web_view_get_page_id (WEBKIT_WEB_VIEW (view)),
+                                                    webkit_web_view_get_uri (WEBKIT_WEB_VIEW (view)),
+                                                    cancellable,
+                                                    (GAsyncReadyCallback)get_best_web_app_icon_cb,
+                                                    g_object_ref (task));
+  } else {
+    g_task_return_boolean (task, FALSE);
+  }
+
+  g_object_unref (task);
+}
+
+gboolean
+ephy_web_view_get_best_web_app_icon_finish (EphyWebView *view,
+                                            GAsyncResult *result,
+                                            gboolean *icon_result,
+                                            char **icon_uri,
+                                            GdkRGBA *icon_color,
+                                            GError **error)
+{
+  GetBestWebAppIconAsyncData *data;
+  GTask *task = G_TASK (result);
+
+  g_return_val_if_fail (g_task_is_valid (result, view), FALSE);
+
+  data = g_task_propagate_pointer (task, error);
+  if (!data)
+    return FALSE;
+
+  if (data->icon_uri != NULL && data->icon_uri[0] != '\0') {
+    *icon_uri = data->icon_uri;
+    data->icon_uri = NULL;
+  }
+
+  if (data->icon_color != NULL && data->icon_color[0] != '\0')
+    gdk_rgba_parse (icon_color, data->icon_color);
+
+  get_best_web_app_icon_async_data_free (data);
+
+  return TRUE;
+}
+
+static void
+get_web_app_title_cb (EphyWebExtensionProxy *web_extension,
+                      GAsyncResult *result,
+                      GTask *task)
+{
+  char *retval;
+  GError *error = NULL;
+
+  retval = ephy_web_extension_proxy_get_web_app_title_finish (web_extension, result, &error);
+  if (!retval)
+    g_task_return_error (task, error);
+  else
+    g_task_return_pointer (task, retval, (GDestroyNotify)g_free);
+  g_object_unref (task);
+}
+
+void
+ephy_web_view_get_web_app_title (EphyWebView *view,
+                                 GCancellable *cancellable,
+                                 GAsyncReadyCallback callback,
+                                 gpointer user_data)
+{
+  GTask *task;
+
+  g_return_if_fail (EPHY_IS_WEB_VIEW (view));
+
+  task = g_task_new (view, cancellable, callback, user_data);
+
+  if (view->priv->web_extension) {
+    ephy_web_extension_proxy_get_web_app_title (view->priv->web_extension,
+                                                webkit_web_view_get_page_id (WEBKIT_WEB_VIEW (view)),
+                                                cancellable,
+                                                (GAsyncReadyCallback)get_web_app_title_cb,
+                                                g_object_ref (task));
+  } else {
+    g_task_return_pointer (task, NULL, NULL);
+  }
+
+  g_object_unref (task);
+}
+
+char *
+ephy_web_view_get_web_app_title_finish (EphyWebView *view,
+                                        GAsyncResult *result,
+                                        GError **error)
+{
+  g_return_val_if_fail (g_task_is_valid (result, view), NULL);
+
+  return g_task_propagate_pointer (G_TASK (result), error);
+}
+
 /**
  * ephy_web_view_get_security_level:
  * @view: an #EphyWebView
