@@ -48,14 +48,6 @@ typedef enum
   EPHY_WEB_VIEW_NAV_FORWARD = 1 << 1
 } EphyWebViewNavigationFlags;
 
-typedef enum
-{
-  EPHY_WEB_VIEW_CHROME_TOOLBAR    = 1 << 0,
-  EPHY_WEB_VIEW_CHROME_DOWNLOADS_BOX = 1 << 1
-} EphyWebViewChrome;
-
-#define EPHY_WEB_VIEW_CHROME_ALL (EPHY_WEB_VIEW_CHROME_TOOLBAR)
-
 typedef enum
 {
   EPHY_WEB_VIEW_STATE_IS_UNKNOWN,
@@ -144,7 +136,13 @@ const char *               ephy_web_view_get_typed_address        (EphyWebView
 void                       ephy_web_view_set_typed_address        (EphyWebView               *view,
                                                                    const char                *address);
 gboolean                   ephy_web_view_get_is_blank             (EphyWebView               *view);
-gboolean                   ephy_web_view_has_modified_forms       (EphyWebView               *view);
+void                       ephy_web_view_has_modified_forms       (EphyWebView               *view,
+                                                                   GCancellable              *cancellable,
+                                                                   GAsyncReadyCallback        callback,
+                                                                   gpointer                   user_data);
+gboolean                  ephy_web_view_has_modified_forms_finish (EphyWebView               *view,
+                                                                   GAsyncResult              *result,
+                                                                   GError                   **error);
 void                       ephy_web_view_get_security_level       (EphyWebView               *view,
                                                                    EphyWebViewSecurityLevel  *level,
                                                                    GTlsCertificate          **certificate,
@@ -162,6 +160,23 @@ void                       ephy_web_view_load_error_page          (EphyWebView
                                                                    const char                *uri,
                                                                    EphyWebViewErrorPage       page,
                                                                    GError                    *error);
+void                       ephy_web_view_get_best_web_app_icon    (EphyWebView               *view,
+                                                                   GCancellable              *cancellable,
+                                                                   GAsyncReadyCallback        callback,
+                                                                   gpointer                   user_data);
+gboolean               ephy_web_view_get_best_web_app_icon_finish (EphyWebView               *view,
+                                                                   GAsyncResult              *result,
+                                                                   gboolean                  *icon_result,
+                                                                   char                     **icon_uri,
+                                                                   GdkRGBA                   *icon_color,
+                                                                   GError                   **error);
+void                       ephy_web_view_get_web_app_title        (EphyWebView               *view,
+                                                                   GCancellable              *cancellable,
+                                                                   GAsyncReadyCallback        callback,
+                                                                   gpointer                   user_data);
+char                      *ephy_web_view_get_web_app_title_finish (EphyWebView               *view,
+                                                                   GAsyncResult              *result,
+                                                                   GError                   **error);
 
 G_END_DECLS
 
