@@ -178,7 +178,11 @@ show_history (GSimpleAction *action,
               GVariant *parameter,
               gpointer user_data)
 {
-  window_cmd_edit_history (NULL, NULL);
+  GtkWindow *window;
+
+  window = gtk_application_get_active_window (GTK_APPLICATION (ephy_shell));
+
+  window_cmd_edit_history (NULL, EPHY_WINDOW (window));
 }
 
 static void
@@ -645,8 +649,6 @@ ephy_shell_get_default (void)
  * @parent_window: the target #EphyWindow or %NULL
  * @previous_embed: the referrer embed, or %NULL
  * @request: a #WebKitNetworkRequest to load or %NULL
- * @chrome: a #EphyEmbedChrome mask to use if creating a new window
- * @is_popup: whether the new window is a popup
  * @user_time: a timestamp, or 0
  *
  * Create a new tab and the parent window when necessary.
@@ -660,8 +662,6 @@ ephy_shell_new_tab_full (EphyShell *shell,
                          EphyEmbed *previous_embed,
                          WebKitURIRequest *request,
                          EphyNewTabFlags flags,
-                         EphyWebViewChrome chrome,
-                         gboolean is_popup,
                          guint32 user_time)
 {
   EphyEmbedShell *embed_shell;
@@ -702,7 +702,7 @@ ephy_shell_new_tab_full (EphyShell *shell,
   if (!in_new_window && parent_window != NULL)
     window = parent_window;
   else
-    window = ephy_window_new_with_chrome (chrome, is_popup);
+    window = ephy_window_new ();
 
   if (flags & EPHY_NEW_TAB_APPEND_AFTER) {
     if (previous_embed) {
@@ -717,7 +717,7 @@ ephy_shell_new_tab_full (EphyShell *shell,
   if (flags & EPHY_NEW_TAB_FIRST)
     position = 0;
 
-  if (flags & EPHY_NEW_TAB_FROM_EXTERNAL) {
+  if (flags & EPHY_NEW_TAB_FROM_EXTERNAL && position == -1) {
     /* If the active embed is blank, use that to open the url and jump to it */
     embed = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (window));
     if (embed != NULL) {
@@ -816,7 +816,7 @@ ephy_shell_new_tab (EphyShell *shell,
 
   embed = ephy_shell_new_tab_full (shell, parent_window,
                                    previous_embed, request, flags,
-                                   EPHY_WEB_VIEW_CHROME_ALL, FALSE, 0);
+                                   0);
 
   if (request)
     g_object_unref (request);
@@ -910,6 +910,10 @@ ephy_shell_get_history_window (EphyShell *shell)
     service = EPHY_HISTORY_SERVICE
       (ephy_embed_shell_get_global_history_service (embed_shell));
     shell->priv->history_window = ephy_history_window_new (service);
+    g_signal_connect (shell->priv->history_window,
+                      "destroy",
+                      G_CALLBACK (gtk_widget_destroyed),
+                      &shell->priv->history_window);
   }
 
   return shell->priv->history_window;
@@ -924,7 +928,9 @@ GObject *
 ephy_shell_get_prefs_dialog (EphyShell *shell)
 {
   if (shell->priv->prefs_dialog == NULL) {
-    shell->priv->prefs_dialog = g_object_new (EPHY_TYPE_PREFS_DIALOG, NULL);
+    shell->priv->prefs_dialog = g_object_new (EPHY_TYPE_PREFS_DIALOG,
+                                              "use-header-bar", TRUE,
+                                              NULL);
     g_signal_connect (shell->priv->prefs_dialog,
                       "destroy",
                       G_CALLBACK (gtk_widget_destroyed),
@@ -1046,6 +1052,7 @@ typedef struct {
   char **uris;
   EphyNewTabFlags flags;
   guint32 user_time;
+  EphyEmbed *previous_embed;
   guint current_uri;
 } OpenURIsData;
 
@@ -1113,16 +1120,16 @@ ephy_shell_open_uris_idle (OpenURIsData *data)
     page_flags = EPHY_NEW_TAB_HOME_PAGE;
   } else {
     page_flags = EPHY_NEW_TAB_OPEN_PAGE;
+    if (data->previous_embed)
+      page_flags |= EPHY_NEW_TAB_APPEND_AFTER;
     request = webkit_uri_request_new (url);
   }
 
   embed = ephy_shell_new_tab_full (ephy_shell_get_default (),
                                    data->window,
-                                   NULL /* parent tab */,
+                                   data->previous_embed,
                                    request,
                                    data->flags | page_flags,
-                                   EPHY_WEB_VIEW_CHROME_ALL,
-                                   FALSE /* is popup? */,
                                    data->user_time);
 
   if (request)
@@ -1130,6 +1137,7 @@ ephy_shell_open_uris_idle (OpenURIsData *data)
 
   data->window = EPHY_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (embed)));
   data->current_uri++;
+  data->previous_embed = embed;
 
   return data->uris[data->current_uri] != NULL;
 }
